﻿namespace Prism.Sample.DynamicModule.ViewModels
{
    using System.ComponentModel;
    using System.Windows.Input;
    using Microsoft.Practices.Prism.Commands;
    using Microsoft.Practices.Prism.Events;
    using Microsoft.Practices.ServiceLocation;
    using Prism.Sample.Infrastructure;

    /// <summary>
    /// Viewmodel for dynamic main view.
    /// </summary>
    public class DynamicMainViewModel
    {
        /// <summary>
        /// Send message command.
        /// </summary>
        private DelegateCommand<object> sendMessageCommand;

        /// <summary>
        /// Prism event aggregator.
        /// </summary>
        private IEventAggregator eventAggregator;

        /// <summary>
        /// Initializes a new instance of the <see cref="DynamicMainViewModel"/> class.
        /// </summary>
        public DynamicMainViewModel()
        {
            if (!DesignerProperties.IsInDesignTool)
            {
                this.eventAggregator = ServiceLocator.Current.GetService(typeof(IEventAggregator)) as IEventAggregator;

                this.sendMessageCommand = new DelegateCommand<object>(o => this.SendMessage_Execute(o));
            }
        }

        /// <summary>
        /// Gets the send message command.
        /// </summary>
        public ICommand SendMessageCommand
        {
            get
            {
                return this.sendMessageCommand;
            }
        }

        /// <summary>
        /// Send message command action.
        /// </summary>
        /// <param name="parameter">The parameter.</param>
        private void SendMessage_Execute(object parameter)
        {
            SendMessageEvent sme = this.eventAggregator.GetEvent<SendMessageEvent>();
            if (sme != null)
            {
                sme.Publish("Hello World");
            }
        }
    }
}
