﻿namespace Prism.Sample.Infrastructure
{
    using System;
    using System.ComponentModel;
    using System.Linq.Expressions;

    /// <summary>
    /// Extension methods for INotifyPropertyChanged.
    /// </summary>
    public static class NotifyPropertyChangedExtensions
    {
        /// <summary>
        /// Raises the PropertyChanged event.
        /// </summary>
        /// <typeparam name="T">The type of source.</typeparam>
        /// <typeparam name="P">The property.</typeparam>
        /// <param name="pc">The property changed event.</param>
        /// <param name="source">The source.</param>
        /// <param name="pe">The property expression.</param>
        public static void Raise<T, P>(this PropertyChangedEventHandler pc, T source, Expression<Func<T, P>> pe)
        {
            if (pc != null)
            {
                pc.Invoke(
                    source,
                    new PropertyChangedEventArgs(((MemberExpression)pe.Body).Member.Name));
            }
        }

        /// <summary>
        /// Raises the PropertyChanged event for all properties.
        /// </summary>
        /// <typeparam name="T">The type of source.</typeparam>
        /// <param name="pc">The property changed event.</param>
        /// <param name="source">The source.</param>
        public static void RaiseAll<T>(this PropertyChangedEventHandler pc, T source)
        {
            if (pc != null)
            {
                pc.Invoke(source, new PropertyChangedEventArgs(string.Empty));
            }
        }
    }
}

