﻿namespace Prism.Sample.OnDemandModule.ViewModels
{
    using System;
    using System.ComponentModel;
    using Microsoft.Practices.Prism.Events;
    using Microsoft.Practices.ServiceLocation;
    using Prism.Sample.Infrastructure;

    /// <summary>
    /// Viewmodel for on demand main view.
    /// </summary>
    public class OnDemandMainViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// Prism event aggregator.
        /// </summary>
        private IEventAggregator eventAggregator;

        /// <summary>
        /// The message.
        /// </summary>
        private string message = "No messages arrived.";

        /// <summary>
        /// Initializes a new instance of the <see cref="OnDemandMainViewModel"/> class.
        /// </summary>
        public OnDemandMainViewModel()
        {
            if (!DesignerProperties.IsInDesignTool)
            {
                this.eventAggregator = ServiceLocator.Current.GetService(typeof(IEventAggregator)) as IEventAggregator;

                SendMessageEvent sme = this.eventAggregator.GetEvent<SendMessageEvent>();
                if (sme != null)
                {
                    sme.Subscribe(this.OtherModule_SendMessage);
                }
            }
        }

        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets the message.
        /// </summary>
        public string Message
        {
            get
            {
                return this.message;
            }

            private set
            {
                this.message = value;
                this.PropertyChanged.Raise(this, o => o.Message);
            }
        }

        /// <summary>
        /// Other module raised 'send message'.
        /// </summary>
        /// <param name="message">The message.</param>
        public void OtherModule_SendMessage(string message)
        {
            this.Message = string.Format("The message '{0}' arrived at {1}.", message, DateTime.Now.ToLocalTime());
        }
    }
}
